@@ -11,8 +11,11 @@ function(input, error.label, ...) {
     else if (is.character(input) && nzchar(input[1])) {
       error.label <- input[1]
     }
+    else {
+      error.label <- NULL
+    }
   }
-  
+
   if (is.character(input)) {
     con <- file(input, encoding = 'UTF-8')
     on.exit(close(con), add = TRUE)
